# -*- coding: utf-8 -*-

"""
Funções para auxiliarem no desenvolvimento
"""

import time

from unicodedata import normalize
from functools import wraps

from repositories.plataforma.models import Conta, Usuario, Contrato
from repositories.cliente.models import Cliente
from li_common.conexoes.email import TemplateEmailSender
from repositories.pedido.models import PedidoVenda


import logging
LOGGER = logging.getLogger()


def tente_outra_vez(excecoes, tentativas=4, tempo_espera=3, multiplicador_espera=1):
    """
    Escuta pelas exceções especificadas e refaz a chamada para o método.
    :param excecoes: As exceções que devem ser escutadas
    :type excecoes: Exception or tuple
    :param tentativas: Número de vezes que deve ser chamado o método
    :type tentativas: int
    :param tempo_espera: Tempo inicial de espera em segundos entre as tentativas
    :type tempo_espera: int
    :param multiplicador_espera: Multiplicador para o tempo de espera nas chamadas subsequentes.
    :type multiplicador_espera: int
    """
    def deco_tente_outra_vez(funcao):
        """
        Decorator para tente_outra_vez
        """
        @wraps(funcao)
        def funcao_tente_outra_vez(*args, **kwargs):
            """
            Tratamento de decorator para tente_outra_vez
            """
            _tentativas, _tempo_espera = tentativas, tempo_espera
            while _tentativas > 1:
                try:
                    return funcao(*args, **kwargs)
                except excecoes, exc:
                    msg = "{}, Reconectando in {} segundos...".format(str(exc), _tempo_espera)
                    if LOGGER:
                        LOGGER.warning(msg)
                    time.sleep(_tempo_espera)
                    _tentativas -= 1
                    _tempo_espera *= multiplicador_espera
            return funcao(*args, **kwargs)
        return funcao_tente_outra_vez
    return deco_tente_outra_vez


def remover_acentos(value):
    """Normalize the values."""
    try:
        return normalize('NFKD', value.decode('utf-8')).encode('ASCII', 'ignore')
    except UnicodeEncodeError:
        return normalize('NFKD', value).encode('ASCII', 'ignore')


def pegar_versao_git():
    """
    pegar_versao_git
    :return: git version
    """
    import subprocess

    try:
        return subprocess.check_output(['git', 'rev-parse', '--short', 'HEAD'])
    except subprocess.CalledProcessError:
        return '0'

def send_email(template_file=None, context=None,
               contrato_id=None, conta_id=None, usuario_id=None, cliente_id=None, pedido_venda_id=None,
               salva_evidencia=True, to_email=None):
    """
    send_email
    :param template_file: template_file
    :param context: context
    :param contrato_id: contrato_id
    :param conta_id: conta_id
    :param usuario_id: usuario_id
    :param cliente_id: cliente_id
    :param pedido_venda_id: pedido_venda_id
    :param salva_evidencia: salva_evidencia
    :param to_email: to_email
    :return: None
    """

    contexto_evidencia = {}
    contexto_evidencia.update(context)

    if cliente_id is not None:
        try:
            cliente = Cliente.objects.get(id=cliente_id)
            context['cliente'] = cliente
        except Cliente.DoesNotExist:
            LOGGER.error("Não foi encontrada Cliente com id={}".format(cliente_id))
            return False

    if pedido_venda_id is not None:
        try:
            pedido_venda = PedidoVenda.objects.get(id=pedido_venda_id)
            context['pedido_venda'] = pedido_venda
            context['objeto'] = pedido_venda.pedido_envio.objeto
        except Cliente.DoesNotExist:
            LOGGER.error("Não foi encontrada PedidoVenda com id={}".format(pedido_venda_id))
            return False

    if conta_id is not None:
        try:
            conta = Conta.objects.get(id=conta_id)
            context['conta'] = conta
        except Conta.DoesNotExist:
            LOGGER.error("Não foi encontrada Conta com id={}".format(conta_id))
            return False

        if context['conta'].contrato_id is not None and contrato_id is None:
            contrato_id = context['conta'].contrato_id

    usuarios = None
    if to_email is None:
        if conta_id is not None or usuario_id is not None:
            kwargs = {}
            if conta_id is not None:
                kwargs['conta_id'] = conta_id

            if usuario_id is not None:
                kwargs['id'] = usuario_id

            usuarios = Usuario.objects.filter(**kwargs)
            LOGGER.debug(usuarios.query)

    # TODO: fazer um if para pegar o contrato_id do primeiro usuario caso nao tenha contrato_id definido
    if contrato_id is not None or usuarios is not None:
        try:
            contrato = Contrato.objects.get(id=contrato_id)
            context['contrato'] = contrato
        except Contrato.DoesNotExist:
            LOGGER.error("Não foi encontrado Contrato com id={}".format(contrato_id))
            return False

    from_email = 'nao-responder@' + context['contrato'].dominio

    if to_email is not None:

        if usuario_id is not None:
            try:
                context["usuario"] = Usuario.objects.get(id=usuario_id)
            except Usuario.DoesNotExist:
                pass

        TemplateEmailSender(template_file, context, from_email, to_email, salva_evidencia, contexto_evidencia, contrato_id, conta_id, usuario_id, pedido_venda_id, cliente_id).send_async()

    else:

        for item in usuarios:
            context['usuario'] = item
            to_email = item.email
            TemplateEmailSender(template_file, context, from_email, to_email, salva_evidencia, contexto_evidencia, contrato_id, conta_id, usuario_id, pedido_venda_id, cliente_id).send_async()
